
!Inputs into this program: data and parameters
!Parameter files: solution_aug23, gamma_step1_212_aug22
!Data files: SIMCE410_final.txt
!            workers_unique.txt
!               mkt_char5.csv



PROGRAM Main_Step2

USE VarDef
USE Datasets


IMPLICIT NONE


integer                   :: APPROX_PROF     !This is an indicator for whether to approximate profits and max the approximated profits or to maximize numerically true profits

integer , parameter       :: Drep=10         !Maximum number draws used: keep the first one for which profit>0
integer, parameter        :: Drep_workpar=1  !number of replicas of workers and parents: just one
integer                   :: Drep_used 
character(150)            :: inputname, outputname, tag
real(8)                   :: gamma_step1(212), rates(n_mkt)
integer                   :: i, npar, idum, sim, noeval, sim_cost, mkt, counter
real(8)                   :: ln_r1(n_mkt)    !these are the rental rates estimated in the previous step

!shocks and types of workers and parents
real(8)                   :: epsilon_pvec(3*n_s*D), epsilon_p(3,n_s,D), epsilon_wvec(4*n_w*D), epsilon_w(4,n_w,D)   !for the interpolation, I keep drawing large numnbers of workers and parents
real(8)                   :: Psi_w(3), Psi_p(3), unif_w(n_w*D), unif_p(n_s*D)
integer                   :: type_ip(n_s,D), type_iw(n_w,D)

!Interpolating profits: obtain one interpolation per market and per shock draw (10=number of regressors, including constant)
real(8)                   :: Reg(iter_num,10), XpX(10,10), XpX_inv(10,10), ahatmat(10,n_mktvd,Dpr),Rsqmat(n_mktvd,Dpr)  !Dpr=number of cost shock draws

!Choosing the lower and upper extremes for the interpolation of ln_r and pv
real(8)                   :: l_lnr, u_lnr, l_p, u_p

!Solutions: one per market and per cost draw  (draw a few shocks, keep only the fist one with positive profits)
          !This is equivalent to choosing a vector of cost shocks that keeps the current number of V schools constant, we are discarding the shocks that are such that at least one school
          !would leave the market. This is an assumption on the distribution of the cost shock, which is assumed to be truncated. 
          !NB profits can be inferred only up to scale, the truncation assumes that the fixed operating costs are zero.
real(8)                   :: pv_omat(n_mktvd, Drep), r_omat(n_mkt,Drep), positive_maxaprof(n_mktvd,Drep), maxaprof_mat(n_mktvd,Drep)

!Given a cost shock and therefore an optimal p and r, compute the quantities coming from the the second stage of the model
!Choices of parents and potential teachers
real(8)                   :: TotwagesM(n_mktvd), SupV(n_mktvd), TSV(n_mktvd), DV(n_mktvd), SM(n_mktvd), EPV(n_mktvd), DM(n_mktvd)
!School finances and gvmt spending
real(8)                   :: gvmtM(n_mktvd), gvmt_Vvoucher(n_mktvd), unit_revenue(n_mktvd), voucher_V(n_mktvd), Tot_fellowships(n_mktvd), unit_fellow(n_mktvd), school_fellow_contrib(n_mktvd), &
                             & gvmtV(n_mktvd), gvmt_Vfellow(n_mktvd), gvmt(n_mktvd)
real(8)                   :: govM_vouchermkt(n_mktvd), govM_extramkt(n_mktvd)
!measures of policy outcome and expenditure
real(8)                   :: statistics(9), mean_ts, mean_tsm, mean_tsv, gov_exp, gov_exp_wages, gov_exp_fell, awm, awv
real(8)                   :: govV_voucher, govV_fellow, govM_voucher, govM_extra




real(8)                   :: avg_lnr(n_mktvd)


!used in simulation, neede this to simulate parents
real(8)                   :: awM_sim(n_mkt), awV_sim(n_mkt), zeroSM_sim(n_mkt), zeroSV_sim(n_mkt)  !average teachers skills in the two school sectors in each market. Indicator for whether the supply of teachers to that sector
                                                                                                   !is zero at that wage rate r

real(8)                   :: pv_oavg(n_mktvd), lr_oavg(n_mktvd), Shocks_used(n_mktvd)

integer                   :: use_sim(Drep)

real(8)                   :: r_used(n_mktvd), pv_used(n_mktvd), maxaprof_used(n_mktvd)

real(8)                   :: unit_fellow_vec(18), unit_revenue_vec(18), true_profit(18), shock_cost(Drep), shock_cost_used(18)

real(8)                   :: overall_gov, overall_govM, overall_govV, overall_wageM, overall_voucherM, overall_extraM, overall_voucherV, overall_fellowV, overall_DM, overall_DV


integer                   :: coeff
real(8)                   :: coeff_dble





!Setting upper and lower bounds for profit interpolation
l_lnr=-4.3900d0; u_lnr=0.30d0   
l_p=0.10d0; u_p=3.80d0        

!Initializing
statistics=-99.0d0




!###################################################################
!##### Read in input parameters and assign thetas using gamma ######
!###################################################################

!gamma is the input into the program. theta is the vector of parameters actually used by the code. the vector gamma must be translated into the vector theta.

!Read in the 4 parameters from step2 of the estimation (theta2, the profit function parameters), then read in the prameters from step1 of the estimation (gamma_step1)
open (unit=7786,file='./solution_aug23')   
   do i=1,4
      read(7786,*) theta2(i)
  end do
close(7786)

gamma_step1=0.0d0
open(unit=1732,file='./gamma_step1_212_aug22')
do i=1,212
   read(1732,*) gamma_step1(i)
end do
close(1732)

!translate gamma_step1 into theta, the parameters of parents and potential teachers
theta=0.0d0


!Assign thetas of workers (the estimated rental rates won't be used because we simulate the optimal (p,r) in the policy experiments)
do i=1,3
  theta(i)=gamma_step1(i)
end do
do i=5,93
 theta(i)=gamma_step1(i-1)
end do
do i=106,109
 theta(i)=gamma_step1(i-13)
end do
 theta(194)=gamma_step1(97)
do i=177, 193
 theta(i)=gamma_step1(i-79)
end do
do i=287,304
 theta(i)=gamma_step1(i-172)
end do

 
!Now set the thetas corresponding to the estimated rental rates to zero so that I don't use the wrong ones. This is to ensure I don't use the estimated rental rates: a zero will give me an error message if used.
do i=177,193
 theta(i)=0.0d0 
end do



!Assign thetas of parents
do i=110, 120
 theta(i)=gamma_step1(i+23)
end do
do i=123,128
 theta(i)=gamma_step1(i+21)
end do
do i=155,157
 theta(i)=gamma_step1(i-5)
end do
do i=162, 165
 theta(i)=gamma_step1(i-9)
end do
 theta(169)=gamma_step1(157)
 theta(171)=gamma_step1(158)
 theta(195)=gamma_step1(159)
do i=197,244
 theta(i)=gamma_step1(i-37)
end do
 theta(247)=gamma_step1(208)
do i=249,252
 theta(i)=gamma_step1(i-40)
end do




!#######################################################
!############## Read in datasets #######################
!######################################################


!SIMCE06_media and primaria together
open(unit=12,file='./SIMCE410_final.txt')
    do i=1,n_s
	       read(12,*) id_s(i), mkt7_s(i),primaria_s(i),rural_s(i), nfam_s(i), peduc_s(i), fem_s(i), unempl_s(i), blue_s(i), mincome_s(i), &
       & m_s(i), v_s(i), mtuition_s(i), standability_s(i) 
    end do
close(12)  


!change wage into 100k pesos. 
do i=1,n_s
  mincome_s(i)=mincome_s(i)/100000.0d0
  if(blue_s(i).eq.2) then
   blue_s(i)=0
  end if 
end do

do i=1,n_s
 if(mkt7_s(i).eq.6661) then
     mkt7_s(i)=1
    else if (mkt7_s(i).eq.6662) then
     mkt7_s(i)=2 
    else if (mkt7_s(i).eq.6663) then
     mkt7_s(i)=3 
    else if (mkt7_s(i).eq.6664) then
     mkt7_s(i)=4 
    else if (mkt7_s(i).eq.6665) then
     mkt7_s(i)=5 
    else if (mkt7_s(i).eq.6666) then
     mkt7_s(i)=6 
    else if (mkt7_s(i).eq.6667) then
     mkt7_s(i)=7 
    else if (mkt7_s(i).eq.6668) then
     mkt7_s(i)=8 
    else if (mkt7_s(i).eq.6669) then
     mkt7_s(i)=9 
    else if (mkt7_s(i).eq.66610) then
     mkt7_s(i)=10 
    else if (mkt7_s(i).eq.66612) then
     mkt7_s(i)=11 
    else if (mkt7_s(i).eq.66613) then
     mkt7_s(i)=12 
    else if (mkt7_s(i).eq.66614) then
     mkt7_s(i)=13 
    else if (mkt7_s(i).eq.66615) then
     mkt7_s(i)=14 
    else if (mkt7_s(i).eq.66616) then
     mkt7_s(i)=15 
    else if (mkt7_s(i).eq.66617) then
     mkt7_s(i)=16 
    else if (mkt7_s(i).eq.66618) then
     mkt7_s(i)=17 
    else if (mkt7_s(i).eq.66619) then
     mkt7_s(i)=18 
   end if
end do

 !### read in the workers' dataset ######
!WORKERS UNIQUE
open(unit=13,file='./workers_unique.txt') !a dot between final and trim indicates that this is the dataset with the corrected weights (corrected using idoneidad and fraction teaching obtained in original casen, before imputation)
    do i=1,n_w
       read(13,*) id_w(1,i), id_w(2,i), mkt7_w(i), graddeg_w(i), perfec_w(i), nkids_w(i), nkids2_w(i), nkids6_w(i), age_w(i), fem_w(i), &
       & peduc_w(i), m_w(i), v_w(i), nt_w(i), h_w(i),wage_w(i)
    end do
close(13)
    
do i=1,n_w
   wage_w(i)=wage_w(i)/100000.0d0
end do

do i=1,n_w
  age_w(i)=age_w(i)-19

   if(mkt7_w(i).eq.6661) then
     mkt7_w(i)=1
    else if (mkt7_w(i).eq.6662) then
     mkt7_w(i)=2 
    else if (mkt7_w(i).eq.6663) then
     mkt7_w(i)=3 
    else if (mkt7_w(i).eq.6664) then
     mkt7_w(i)=4 
    else if (mkt7_w(i).eq.6665) then
     mkt7_w(i)=5 
    else if (mkt7_w(i).eq.6666) then
     mkt7_w(i)=6 
    else if (mkt7_w(i).eq.6667) then
     mkt7_w(i)=7 
    else if (mkt7_w(i).eq.6668) then
     mkt7_w(i)=8 
    else if (mkt7_w(i).eq.6669) then
     mkt7_w(i)=9 
    else if (mkt7_w(i).eq.66610) then
     mkt7_w(i)=10 
    else if (mkt7_w(i).eq.66612) then
     mkt7_w(i)=11 
    else if (mkt7_w(i).eq.66613) then
     mkt7_w(i)=12 
    else if (mkt7_w(i).eq.66614) then
     mkt7_w(i)=13 
    else if (mkt7_w(i).eq.66615) then
     mkt7_w(i)=14 
    else if (mkt7_w(i).eq.66616) then
     mkt7_w(i)=15 
    else if (mkt7_w(i).eq.66617) then
     mkt7_w(i)=16 
    else if (mkt7_w(i).eq.66618) then
     mkt7_w(i)=17 
    else if (mkt7_w(i).eq.66619) then
     mkt7_w(i)=18 
   end if
end do


!MKT_CHAR
open(unit=16,file='./mkt_char5.csv')
    do i=1,n_mktvd
       read(16,*) mkt7(i),  size_mkt7(i), educ_mkt7(i),   sdeduc_mkt7(i), blue_mkt7(i),mincome_mkt7(i), sdminc_mkt7(i), fona_mkt7(i), fonb_mkt7(i), fonc_mkt7(i),&
                 & fond_mkt7(i),isapre_mkt7(i), chilesol_mkt7(i), rural_mkt7(i), unempl_mkt7(i), nowork_mkt7(i), nsub_mkt7(i), r_13(i), r_15(i), Norte(i),&
                 & Central(i), Sur(i), numM(i), numV(i), numU(i),Drural_mkt7(i), Dchilsol_mkt7(i),studM_mkt7(i),studV_mkt7(i),sharestudV_mkt7(i), teachM_mkt7(i),&
                 & imp_teachV_mkt7(1,i),imp_teachV_mkt7(2,i),imp_teachV_mkt7(3,i),imp_teachV_mkt7(4,i),imp_teachV_mkt7(5,i),imp_teachV_mkt7(6,i),imp_teachV_mkt7(7,i), &
                 & imp_teachV_mkt7(8,i),imp_teachV_mkt7(9,i),imp_teachV_mkt7(10,i),imp_shareteachV_mkt7(1,i),imp_shareteachV_mkt7(2,i),imp_shareteachV_mkt7(3,i),&
                 & imp_shareteachV_mkt7(4,i),imp_shareteachV_mkt7(5,i),imp_shareteachV_mkt7(6,i),imp_shareteachV_mkt7(7,i),imp_shareteachV_mkt7(8,i),&
                 & imp_shareteachV_mkt7(9,i),imp_shareteachV_mkt7(10,i), imp_studteachV(1,i),imp_studteachV(2,i),imp_studteachV(3,i),imp_studteachV(4,i),imp_studteachV(5,i),&
                 & imp_studteachV(6,i),imp_studteachV(7,i),imp_studteachV(8,i),imp_studteachV(9,i),imp_studteachV(10,i), studteachM(i), mean_maxpv(i)
    end do  
close(16)

!Market level categories
!pvd_cat, educd_cat, mincomed_cat
do i=1,n_mkt
 !pvd_cat
 if(mean_maxpv(i)<=40000.0d0.AND.mean_maxpv(i)>=0.0d0) then
   pvd_cat(i)=1
 else if  (mean_maxpv(i)<=48000.0d0.AND.mean_maxpv(i)>40000.0d0) then
   pvd_cat(i)=2
 else if (mean_maxpv(i)<=53500.0d0.AND.mean_maxpv(i)>48000.0d0 ) then
   pvd_cat(i)=3
 else if (mean_maxpv(i)>53500.0d0) then
   pvd_cat(i)=4
 else
   pvd_cat(i)=-99
 end if
 
 !educd_cat
 if(educ_mkt7(i)<=8.60d0.AND.educ_mkt7(i)>=0.0d0) then
  educd_cat(i)=1
 else if (educ_mkt7(i)> 8.60d0.AND.educ_mkt7(i)<=9.30d0) then
  educd_cat(i)=2
 else if (educ_mkt7(i)>9.30d0.AND.educ_mkt7(i)<=10.10d0) then
  educd_cat(i)=3
 else if (educ_mkt7(i)>10.10d0) then
  educd_cat(i)=4
 else
  educd_cat(i)=-99
 end if
 
 !mincomed_cat
 if(mincome_mkt7(i)<=190000.0d0.AND.mincome_mkt7(i)>=0.0d0) then
  mincomed_cat(i)=1
 else if(mincome_mkt7(i)>190000.0d0.AND.mincome_mkt7(i)<=211358.0d0) then
  mincomed_cat(i)=2
 else if (mincome_mkt7(i)>211358.0d0.AND.mincome_mkt7(i)<=250000.0d0) then
  mincomed_cat(i)=3
 else if (mincome_mkt7(i)>250000.0d0) then
  mincomed_cat(i)=4
 else
  mincomed_cat(i)=-99
 end if
 
 !Dblue dummy
 if(blue_mkt7(i).le.0.6490d0) then
  Dblue_mkt7(i)=0
 else
  Dblue_mkt7(i)=1
 end if 
end do

                 
!All monetary variables (individual and market level) are in terms of 100k pesos

do i=1,n_mkt
  mincome_mkt7(i)=mincome_mkt7(i)/100000.0d0
  mean_maxpv(i)=mean_maxpv(i)/100000.0d0
end do
do i=1,n_mkt
 if(rural_mkt7(i)>=.22) then
  Drural_mkt7(i)=1.0d0
 else
  Drural_mkt7(i)=0.0d0
 end if
 if(chilesol_mkt7(i)>=0.0128) then
  Dchilsol_mkt7(i)=1.0d0
 else
  Dchilsol_mkt7(i)=0.0d0
 end if    
end do

CALL cpu_time (EndTime)
!write(*,*) "Time:", EndTime-StartTime
MidTime=EndTime







!#######################################################################################################################
!Draw parents' and workers' shocks and types. Will need this if I interpolate profits (faster to draw them ouside ######
!of the profit interpolation loop ######################################################################################
!#######################################################################################################################


!Parents' shocks
	idum=7777
	call gasdev(idum,epsilon_pvec,3*n_s*D)
	do i=1,n_s
 	 do sim=1,D
  	   epsilon_p(1,i,sim)=exp(theta(156))*epsilon_pvec(i+(sim-1)*n_s)          !production of cogn ach in M
  	   epsilon_p(2,i,sim)=exp(theta(157))*epsilon_pvec(n_s*D+i+(sim-1)*n_s)    !production of cogn ach in V
  	   epsilon_p(3,i,sim)=exp(theta(155))*epsilon_pvec(2*n_s*D+i+(sim-1)*n_s)  !preference for M


	   epsilon_pglobal(1,i,sim)=epsilon_p(1,i,sim)
  	   epsilon_pglobal(2,i,sim)=epsilon_p(2,i,sim)
  	   epsilon_pglobal(3,i,sim)=epsilon_p(3,i,sim)


 	 end do 
	end do


!Workers' shocks
	! Draw D draws for every worker, and shock type
	idum=3333
	call gasdev(idum,epsilon_wvec,4*n_w*D)

	do i=1,n_w
	 do sim=1,D
	  epsilon_w(1,i,sim)=exp(theta(106))*epsilon_wvec(i+(sim-1)*n_w)            !preference for Home
	  epsilon_w(2,i,sim)=exp(theta(107))*epsilon_wvec(n_w*D+i+(sim-1)*n_w)   !shock in M wage
	  epsilon_w(3,i,sim)=exp(theta(108))*epsilon_wvec(2*n_w*D+i+(sim-1)*n_w) !shock in V wage
	  epsilon_w(4,i,sim)=exp(theta(109))*epsilon_wvec(3*n_w*D+i+(sim-1)*n_w) !shock in NT wage

         epsilon_wglobal(1,i,sim)=epsilon_w(1,i,sim)
          epsilon_wglobal(2,i,sim)=epsilon_w(2,i,sim)
          epsilon_wglobal(3,i,sim)=epsilon_w(3,i,sim)
          epsilon_wglobal(4,i,sim)=epsilon_w(4,i,sim)
	 end do
	end do  


!Workers' type probabilities, use them to draw workers' types
	!Draw one type draw for each individual and simulation
	Psi_w(1)=1.0d0-theta(92)-theta(93)
	Psi_w(2)=theta(92)
	Psi_w(3)=theta(93)

	!write(*,*) "Prob1, prob2, prob3", Psi(1), Psi(2), Psi(3)

	idum=777
	unif_w=-99.0d0
	call ran1(idum,unif_w,n_w*D)
	do i=1,n_w
	  do sim=1,D
	    if(unif_w(i+(sim-1)*D)>=0.0d0.AND.unif_w(i+(sim-1)*D)<=Psi_w(1)) then
	      type_iw(i,sim)=1
             type_iwglobal(i,sim)=1
	    else if(unif_w(i+(sim-1)*D)>Psi_w(1).AND.unif_w(i+(sim-1)*D)<=Psi_w(1)+Psi_w(2)) then
	      type_iw(i,sim)=2
             type_iwglobal(i,sim)=2
	    else if (unif_w(i+(sim-1)*D)>Psi_w(1)+Psi_w(2).AND.unif_w(i+(sim-1)*D)<=1.0d0) then
	      type_iw(i,sim)=3
             type_iwglobal(i,sim)=3
	    end if
	   end do
	end do
	
!Parents' type probabilities, use them to draw workers' types
	!Draw one type draw for each individual
	Psi_p(1)=(1.0d0-theta(127)-theta(128))
	Psi_p(2)=theta(127)
	Psi_p(3)=theta(128)

	idum=775
	unif_p=-99.0d0
	call ran1(idum,unif_p,n_s*D)

	do i=1,n_s
	  do sim=1,D
	    if(unif_p(i+(sim-1)*D)>=0.0d0.AND.unif_p(i+(sim-1)*D)<=Psi_p(1)) then
	      type_ip(i,sim)=1
             type_ipglobal(i,sim)=1
	    else if(unif_p(i+(sim-1)*D)>Psi_p(1).AND.unif_p(i+(sim-1)*D)<=Psi_p(1)+Psi_p(2)) then
	      type_ip(i,sim)=2
	      type_ipglobal(i,sim)=2
	    else if (unif_p(i+(sim-1)*D)>Psi_p(1)+Psi_p(2).AND.unif_p(i+(sim-1)*D)<=1.0d0) then
	      type_ip(i,sim)=3
	      type_ipglobal(i,sim)=3
	    end if
	   end do
	end do


!write all shocks to file, so that in counterfactuals I use the same shocks

open(unit=87857,file='./baseline_shocks_parents.txt')
  do i=1,n_s
    do sim=1,D
	 
	   write(87857,*) epsilon_p(1,i,sim), epsilon_p(2,i,sim),  epsilon_p(3,i,sim), type_ip(i,sim)


    end do
  end do

close(87857)


open(unit=878571,file='./baseline_shocks_workers.txt')
  do i=1,n_w
    do sim=1,D
	 
	   write(878571,*) epsilon_w(1,i,sim), epsilon_w(2,i,sim), epsilon_w(3,i,sim), epsilon_w(4,i,sim), type_iw(i,sim)


    end do
  end do

close(878571)


!####################################################################################
! Maximize profits. Either interpolate them and max the approximated profits or #####
! maximize numericlaly true profits, after fixing the price at the cap ##############
!####################################################################################

APPROX_PROF=0
IF (APPROX_PROF.eq.1) THEN
write(*,*) ""
write(*,*) "Interpolate profits and compute optimal prices in each market and cost shock draw"

! Interpolate profits ########################################



!The matrix Reg has a number of lines equal to the number of observations on prices and rates used in the interpolation, and a number of columns equal to 10, the number of regressors
!used in the interpoalting polynomial
!The regressors are:
!     Reg(iter,1)=constant
!     Reg(iter,2)=pv                          !pv in terms of 100mila pesos
!     Reg(iter,3)=Reg(iter,2)**2.0d0          !pv^2
!     Reg(iter,4)=exp(ln_r)!exp(dble(lnr_iter)/100.0d0) !r (on wage expressed in millions)
!     Reg(iter,5)=Reg(iter,4)**2.0d0          !r^2
!     Reg(iter,6)=Reg(iter,2)*Reg(iter,4)     !pv*r
!     Reg(iter,7)=Reg(iter,2)**3.0d0          !pv^3
!     Reg(iter,8)=Reg(iter,4)**3.0d0          !r^3
!     Reg(iter,9)=Reg(iter,3)*Reg(iter,4)     !pv^2 * r
!     Reg(iter,10)=Reg(iter,2)*Reg(iter,5)    !pv * r^2


call  assign_regressors(l_lnr, u_lnr, l_p, u_p, Reg)  !It only builds the vector of regressors Reg. It does not do XpX anymore, bc that depends on the number of misisng obs.

  

call Interpolation(Drep,l_lnr, u_lnr, l_p, u_p,outputname,Reg,ahatmat,Rsqmat,epsilon_p,epsilon_w,type_ip, type_iw, pv_omat, r_omat,positive_maxaprof,maxaprof_mat,shock_cost)

write(*,*) ""
write(*,*) "optimal r's under cost shock 1"
write(*,*) r_omat(1:18,1)
write(*,*) "optimal r's under cost shock 2"
write(*,*) r_omat(1:18,2)
write(*,*) "optimal r's under cost shock 3"
write(*,*) r_omat(1:18,3)




ELSE

!############ Maximize profits numerically, I know prices are at cap ##########

write(*,*) ""
write(*,*) "Maximize true profits numerically wrt r, at the price cap"

pv_omat=p_cap
call find_optimal(Drep, pv_omat, r_omat, maxaprof_mat)




write(*,*) ""
write(*,*) r_omat
write(*,*) "optimal r's MATRIX"

END IF !Approx_prof


!########### On average across mkts, what are the optimal prices? ################

!Now compute average prices charged in each market by taking an across shocks average
pv_oavg=0.0d0
lr_oavg=0.0d0
Shocks_used=0.0d0

do sim=1,Drep
  do mkt=1,n_mkt
   if(pv_omat(mkt,sim)>0.0d0.AND.r_omat(mkt,sim)>0.0d0) then
     pv_oavg(mkt)=pv_oavg(mkt)+pv_omat(mkt,sim)
     lr_oavg(mkt)=lr_oavg(mkt)+log(r_omat(mkt,sim))
     Shocks_used(mkt)=Shocks_used(mkt)+1.0d0
     
   end if
  end do
end do

!write(*,*) "averag tuition and log wage rates in each market"
 do mkt=1,n_mkt
  pv_oavg(mkt)=pv_oavg(mkt)/Shocks_used(mkt)
  lr_oavg(mkt)=lr_oavg(mkt)/Shocks_used(mkt)
!  write(*,*) mkt, pv_oavg(mkt), lr_oavg(mkt)
end do


!######################################################################################
!############# SIMULATION: SIMULATE PARENTS AND WORKERS AT THE OPTIMAL PRICES #########
!######################################################################################
!This part of the code:
!    - picks an optimal p and r
!    - writes the optimal profits at that p and r to a file called optimal_profits_baseline.txt
!    - simulates datasets of workers and parents

!For each market, pick an error draw that is such that profits are positive
pv_used=-99.0d0
r_used=-99.0d0

do mkt=1,n_mktvd
   sim =1
  do while (sim<=Drep)
     if(pv_omat(mkt,sim)>0.0d0.AND.r_omat(mkt,sim)>0.0d0.AND.maxaprof_mat(mkt,sim)>0.0d0) then
       pv_used(mkt)=pv_omat(mkt,sim)
       r_used(mkt)=r_omat(mkt,sim)
       shock_cost_used(mkt)=shock_cost(sim)
       maxaprof_used(mkt)=maxaprof_mat(mkt,sim)
       sim=Drep+1  !this is telling it to exit from the loop
     else
      sim=sim+1
     end if
  end do !while
end do


write(*,*) "Optimal p and r by market"
do mkt=1,n_mkt
  write(*,*) mkt, pv_used(mkt), r_used(mkt)
end do


open(unit=878090,file='./optimal_profits_baseline.txt')
do mkt=1,18
  write(878090,*) maxaprof_used(mkt)
end do
close(878090)


   
	 
	 
open(unit=4658,file='./prices_profits_baseline.txt', access='append')
write(4658,*) "mkt", "p", "r", "maximized profits"

do mkt=1,n_mkt

  write(4658,'(1I5,3F10.2)') mkt, pv_used(mkt), r_used(mkt),  maxaprof_used(mkt)

end do
close(4658)





write(*,*) ""
write(*,*) "Simulating workers"
call simdata_workers2_gvmt(statistics,r_used, awM_sim, awV_sim, zeroSM_sim, zeroSV_sim,TotwagesM, SupV, TSV)


!Now simulate parents given workers' optimal sorting and school's optimal price

write(*,*) "average teaching skills by mkt, M and V"
do mkt=1,18
  write(*,*) mkt, awM_sim(mkt),awV_sim(mkt)
end do
write(*,*) ""
write(*,*) "Simulating parents"
call simdata_simce_gvmt(statistics,zeroSM_sim, zeroSV_sim, awM_sim,awV_sim, pv_used,DV,DM , EPV)


!Now compute the ACTUAL PROFITS, not the APPROXIMATED ONES
do mkt=1,18

       if(DV(mkt).ne.0.0d0.AND.SupV(mkt).ne.0.0d0) then
           !write profit function using actual governmental formulas
           if(0.50d0*oneUSE<EPV(mkt).le.oneUSE) then
              unit_revenue_vec(mkt)=pv_used(mkt)-0.10d0*(EPV(mkt)-0.50d0*oneUSE)
           else if (oneUSE<EPV(mkt).le.2.0d0*oneUSE) then
              unit_revenue_vec(mkt)=pv_used(mkt)-0.10d0*0.50d0*oneUSE-0.20d0*(EPV(mkt)-oneUSE)
           else if (2.0d0*oneUSE<EPV(mkt))  then
              unit_revenue_vec(mkt)=pv_used(mkt)-0.10d0*0.50d0*oneUSE-0.20d0*oneUSE-0.350d0*(EPV(mkt)-2.0d0*oneUSE)
           end if 
        
           if(EPV(mkt).le.oneUSE) then
             unit_fellow_vec(mkt)=0.050d0*EPV(mkt)
           else if (oneUSE<EPV(mkt).le.2.0d0*oneUSE) then
             unit_fellow_vec(mkt)=0.050d0*oneUSE+0.070d0*(EPV(mkt)-oneUSE)
           else if(2.0d0*oneUSE<EPV(mkt)) then
             unit_fellow_vec(mkt)=0.050d0*oneUSE+0.070d0*oneUSE+0.10d0*(EPV(mkt)-2.0d0*oneUSE)
           end if

          true_profit(mkt)=    (unit_revenue_vec(mkt)-unit_fellow_vec(mkt))*DV(mkt)-(theta2(2)+ shock_cost_used(mkt))*DV(mkt)-theta2(3)*DV(mkt)**2.0d0&
                                   &-(exp(r_used(mkt))*TSV(mkt))-theta2(4)*((DV(mkt)/SupV(mkt))/45.0d0)    

      else
         true_profit(mkt)=-99.0d0
      end if

end do

open(unit=878,file='./true_profits_baseline.txt')
 do mkt=1,18
  write(878,*) true_profit(mkt)
end do
close(878)






write(*,*) ""
write(*,*) "Simulating government spending "
!######################################################################################
!############# SIMULATION: GOVERNMENT SPENDING ########################################
!######################################################################################
! This part of the code computes government spending. It writes the total expenditures by mkt and 
! a break down of those expenses to the file gov_exp_breakdown.txt, it writes total
! expenditure natioanlly onto the file gvmt_exp.txt
! Finally, it writes onto a file called statistics.txt the gov expenditures and test scores.



!First, do government spending in the Municipal school
gvmtM=0.0d0

do mkt=1,n_mkt
  if(DM(mkt)*voucher-TotwagesM(mkt)<0.d0) then !!! Must pay also the difference between revenues and costs

 !write(*,*) "Negative profits in M school in mkt", mkt !13 out of 18 markets, voucher revenuws are not enough to cover wages!

    gvmtM(mkt)=DM(mkt)*voucher+(TotwagesM(mkt)-DM(mkt)*voucher )
 else
   gvmtM(mkt)=DM(mkt)*voucher
 end if
 
end do



gvmt_Vvoucher=0.0d0

   do mkt=1,n_mkt

         gvmt_Vvoucher(mkt)=voucher*DV(mkt)
   end do




!Compute total fellowships in the V school by market

Tot_fellowships=0.0d0
do mkt=1,n_mkt 
   Tot_fellowships(mkt)=(pv_used(mkt)-voucher)*DV(mkt)-EPV(mkt)*DV(mkt)
 !  write(*,*) mkt, Tot_fellowships(mkt)
end do

!Now compute school's contribution to the fellowship
unit_fellow=0.0d0
school_fellow_contrib=0.0d0
do mkt=1,n_mkt
  
     gvmt_Vfellow(mkt)=0.06821*EPV(mkt)*DV(mkt)


gvmtV(mkt)=gvmt_Vfellow(mkt)+gvmt_Vvoucher(mkt)


gvmt(mkt)=gvmtM(mkt)+gvmtV(mkt)
end do


 govM_voucher=0.0d0
 govM_extra=0.0d0
govM_extramkt=0.0d0

do mkt=1,18
  govM_voucher=govM_voucher+DM(mkt)*voucher
  govM_vouchermkt(mkt)=DM(mkt)*voucher
if(gvmtM(mkt)-DM(mkt)*voucher>0.0d0) then
  govM_extra=govM_extra+gvmtM(mkt)-DM(mkt)*voucher
 govM_extramkt(mkt)=gvmtM(mkt)-DM(mkt)*voucher
end if
 
end do


!Now write to file all the market level values for government expenditure!!!!!!

open(unit=548781,file='./gov_exp_breakdown_baseline.txt')
write(548781,*) "gov, govM, govV, wagesM, vocherM, extraM, voucherV, fellowV"
do mkt=1,n_mktvd
  write(548781,'(8F10.2)'   ) gvmt(mkt), gvmtM(mkt) , gvmtV(mkt) , TotwagesM(mkt), govM_vouchermkt(mkt), govM_extramkt(mkt), gvmt_Vvoucher(mkt), gvmt_Vfellow(mkt)
end do


close(548781)


!now write the enrollment in each school

open(unit=4529,file='./enrollment_baseline.txt')
write(4529,*) "DM, DV"
do mkt=1,18
  write(4529,*) DM(mkt), DV(mkt)
end do
close(4529)


 govV_voucher=0.0d0
do mkt=1,18
 govV_voucher=govV_voucher+gvmt_Vvoucher(mkt)   
end do

 govV_fellow=0.0d0
do mkt=1,18
  govV_fellow= govV_fellow+gvmt_Vfellow(mkt)
end do



write(*,*) "writing gvmt_exp.txt"
open(unit=76576,file='gvmt_exp_baseline.txt')
write(76576,*) "tot gvmt exp, gvmt exp M, gvmt exp V"
do mkt=1,n_mkt
  write(76576,*) gvmt(mkt), gvmtM(mkt), gvmtV(mkt)

end do
write(76576,*) ""
write(76576,*) "TOT GVMT EXPENDITURE AT BASELINE"
write(76576,*) gvmt(1)+gvmt(2)+gvmt(3)+gvmt(4)+gvmt(5)+gvmt(6)+gvmt(7)+gvmt(8)+gvmt(9)+gvmt(10)+gvmt(11)+gvmt(12)+gvmt(13)+gvmt(14)+gvmt(15)+gvmt(16)+gvmt(17)+gvmt(18)

close(76576)


overall_gov=0.0d0; overall_govM=0.0d0; overall_govV=0.0d0; overall_wageM=0.0d0; overall_voucherM=0.0d0; overall_extraM=0.0d0; overall_voucherV=0.0d0; overall_fellowV=0.0d0
overall_DM=0.0d0; overall_DV=0.0d0

do mkt=1,18
  overall_gov=overall_gov+gvmt(mkt)
  overall_govM=overall_govM+gvmtM(mkt)
  overall_govV=overall_govV+gvmtV(mkt)
  overall_wageM=overall_wageM+TotwagesM(mkt)
  overall_voucherM=overall_voucherM+govM_vouchermkt(mkt)
  overall_extraM=overall_extraM+govM_extramkt(mkt)
  overall_voucherV=overall_voucherV+gvmt_Vvoucher(mkt)
  overall_fellowV=overall_fellowV+gvmt_Vfellow(mkt)
  overall_DM=overall_DM+DM(mkt)
  overall_DV=overall_DV+DV(mkt)
end do



write(*,*) "writing gvmt_exp_totals.txt"
open(unit=765761,file='gvmt_exp_baseline_totals.txt')
write(765761,*) "overall_gov,overall_govM,overall_govV,overall_wageM,overall_voucherM,overall_extraM,overall_voucherV,overall_fellowV, overall_DM, overall_DV"

  write(765761,*) overall_gov,overall_govM,overall_govV,overall_wageM,overall_voucherM,overall_extraM,overall_voucherV,overall_fellowV,overall_DM, overall_DV




close(765761)






write(*,*) "writing to file optimal r's"

open(unit=76767,file='./optimal_r_baseline.txt')
  do mkt=1,n_mktvd
  write(76767,*) r_used(mkt)
  end do
close(76767)

statistics(4)=0.0d0
statistics(5)=0.0d0
statistics(6)=0.0d0
do mkt=1,n_mktvd

  statistics(4)=statistics(4)+gvmt(mkt)
   statistics(5)=statistics(5)+TotwagesM(mkt)
  statistics(6)=statistics(6)+gvmt_Vfellow(mkt)
end do


open(unit=3876,file='./statistics_baseline.txt')

write(3876,*) "mean test scores, mean test scores in M, mean test scores in V, gov exp, gov exp in wages, gov exp in fello, mean abilities teachers, mean abilities M teachers,&
&  mean abilities V teachers, expenditure in M vouchers, expenditure in M extra, expenditure in V vouchers"
write(3876,'(12e14.4)') statistics(1:9), govM_voucher, govM_extra, govV_voucher
close(unit=3876)

open(unit=334, file='./demand_enrollment_baseline.txt')

write(334,*) "mkt, DM, DV"
do mkt=1,18
write(334,*) mkt, DM(mkt), DV(mkt)
end do
close(334)





















END PROGRAM Main_Step2